/*
 * Copyright 2016 HuntBugs contributors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package one.util.huntbugs.detect;

import com.strobel.decompiler.ast.AstCode;
import com.strobel.decompiler.ast.Expression;

import one.util.huntbugs.registry.MethodContext;
import one.util.huntbugs.registry.anno.AstNodes;
import one.util.huntbugs.registry.anno.AstVisitor;
import one.util.huntbugs.registry.anno.WarningDefinition;
import one.util.huntbugs.util.Exprs;
import one.util.huntbugs.util.Nodes;

/**
 * @author Tagir Valeev
 *
 */
@WarningDefinition(category = "BadPractice", name = "AverageComputationCouldOverflow", maxScore = 35)
public class AverageComputation {
    @AstVisitor(nodes = AstNodes.EXPRESSIONS)
    public void visitExpression(Expression expr, MethodContext mc) {
        if (expr.getCode() != AstCode.LoadElement && expr.getCode() != AstCode.StoreElement)
            return;
        Expression idx = Exprs.getChild(expr, 1);
        if (!isDiv2(idx))
            return;
        Expression orig = Exprs.getChild(idx, 0);
        if (orig.getCode() != AstCode.Add)
            return;
        mc.report("AverageComputationCouldOverflow", 0, idx);
    }

    private boolean isDiv2(Expression idx) {
        return (idx.getCode() == AstCode.Div && Integer.valueOf(2).equals(Nodes.getConstant(idx.getArguments().get(1))))
            || (idx.getCode() == AstCode.Shr && Integer.valueOf(1).equals(Nodes.getConstant(idx.getArguments().get(1))));
    }
}
